﻿package com 
{
	import com.objects.AdvancedPhysicsObject;
	import flash.events.EventDispatcher;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import org.papervision3d.core.proto.LightObject3D;
	
	import org.papervision3d.core.math.Number3D;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.shadematerials.FlatShadeMaterial;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.objects.primitives.Sphere;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.lights.PointLight3D;

	import com.events.LevelLoadEvent;
	
	import jiglib.geometry.JBox;
	import jiglib.math.JMatrix3D;
	import jiglib.math.JNumber3D;
	
	/**
	 * LevelBuilder - builds the levels of the game from XML files
	 * 
	 * @author Devin Reimer - blog.almostlogical.com
	 */ 
	public class LevelBuilder extends EventDispatcher
	{
		private static const MARKER_BOX_TYPE = "markerbox";	
			
		private var _physicsObjArray:Array;
		private var _playerResetNumber3D:Number3D;
		private var _playerResetRotationY:Number;
		
		private var xmlLoader:URLLoader;
		private var lightSource:PointLight3D;
		private var currentLevel:int;
		
		public function LevelBuilder(theLightSource:PointLight3D) 
		{
			lightSource = theLightSource;
		}
		
		public function get physicsObjArray():Array { return _physicsObjArray; }
		public function get playerResetNumber3D():Number3D { return _playerResetNumber3D; }
		public function get playerResetRotationY():Number { return _playerResetRotationY; }
		
		public function loadLevelXML(xmlFileName:String):void
		{
			currentLevel = 1;
			xmlLoader = new URLLoader();
			xmlLoader.load(new URLRequest(xmlFileName));
			xmlLoader.addEventListener(Event.COMPLETE,onXMLLoadComplete,false,0,true);
			xmlLoader.addEventListener(IOErrorEvent.IO_ERROR, onLoadFailure,false,0,true);
		}
		
		private function loadComplete():void
		{
			dispatchEvent(new LevelLoadEvent(LevelLoadEvent.LEVEL_LOAD_COMPLETE));
		}
		
		//loads in an xml to build the level
		private function buildLevelUsingXML(theXML:XML):void
		{
			var levelXML:XMLList;
			var groupNumber3D:Number3D;
			var groupName:String;
			
			levelXML = theXML.level.(@id == currentLevel);
			
			_playerResetNumber3D = new Number3D(levelXML.@playerX, levelXML.@playerY, levelXML.@playerZ);
			_playerResetRotationY = Number(levelXML.@playerRotationY);

			_physicsObjArray = new Array();
			
			for each (var groupXML:XML in levelXML.group) //goes through each group
			{
				groupNumber3D = new Number3D(groupXML.@x, groupXML.@y, groupXML.@z);
				groupName = groupXML.@name;
				
				for each (var physicsObjXML:XML in groupXML.physicsObj)
				{
					_physicsObjArray.push(createObjFromXML(physicsObjXML,groupName,groupNumber3D));
				}
			}
		
			loadComplete();
		}
		
		private function createObjFromXML(objXML:XML, groupName:String, groupNumber3D:Number3D):AdvancedPhysicsObject
		{
			var basicPropXML:XMLList = objXML.basicProp;
			var objMatList:MaterialsList = new MaterialsList();
			var objSkin:Cube;
			var objColour:uint = objXML.@colour;
			var objJBox:JBox;
			var objJBoxWidth:int = basicPropXML.@width;
			var objJBoxDepth:int = basicPropXML.@depth;
			var objJBoxHeight:int = basicPropXML.@height;
			var objJBoxNumber3D:Number3D = new Number3D(int(basicPropXML.@x) + (objJBoxWidth/2), int(basicPropXML.@y) - (objJBoxHeight/2), int(basicPropXML.@z) + (objJBoxDepth/2)); //this puts starting point at the top, back, left corner
			var objJBoxMass:int = objXML.@mass;
			var objType:String = objXML.@type;
			var objName:String = groupName + " - " + objXML.@name;
			var segmentsS:int = 1;
			var segmentsT:int = 1;
			var segmentsH:int = 1;
			var isMovable:Boolean = stringToBoolean(objXML.@isMovable);
			var isResettable:Boolean = stringToBoolean(objXML.@isResettable);
			var isMarker:Boolean = false;
			var advancedPhysicsObj:AdvancedPhysicsObject;
			
			objJBoxNumber3D.plusEq(groupNumber3D); //addes this to group
				
			if (objJBoxWidth > 250 || objJBoxDepth > 250 || objJBoxHeight>250) //if object is large increase number of triangles
			{
				segmentsS = 3;
				segmentsT = 3;
				segmentsH = 3;
			}
		
			objMatList.addMaterial(new FlatShadeMaterial(lightSource,objColour),"all");
			objSkin = new Cube(objMatList, objJBoxWidth, objJBoxDepth, objJBoxHeight,segmentsS,segmentsT,segmentsH);
			
			objJBox = new JBox(objSkin, isMovable, objJBoxWidth, objJBoxDepth, objJBoxHeight);
			objJBox.MoveTo(new JNumber3D(objJBoxNumber3D.x, objJBoxNumber3D.y, objJBoxNumber3D.z), JMatrix3D.IDENTITY);
		
			if (isMovable)
			{
				if (objJBoxMass <= 0)
				{
					objJBoxMass = 1;
				}
				objJBox.setMass(objJBoxMass);
				if (objType == MARKER_BOX_TYPE)
				{
					isMarker = true;
				}
			}
			
			advancedPhysicsObj = new AdvancedPhysicsObject(objJBox, objName,objJBoxNumber3D, isMarker, isResettable);
			
			return advancedPhysicsObj;
		}
			
		private function onXMLLoadComplete(evt:Event):void
		{
			var theXML:XML;
			try
			{
				XML.ignoreWhitespace = true;
				theXML = XML(evt.target.data);
			}
			catch(error:Error)
			{
				trace("Error In XML (General)- " + error.message);
			}
			
			xmlLoader.removeEventListener(Event.COMPLETE,onXMLLoadComplete,false);
			xmlLoader.removeEventListener(IOErrorEvent.IO_ERROR, onLoadFailure, false);
			xmlLoader = null;
			
			buildLevelUsingXML(theXML);
			
		}
		private function onLoadFailure(evt:Event):void
		{
			trace("Load Failure: " + evt);
		}
		
		private function stringToBoolean(valStr:String):Boolean
		{
		  return valStr.toLowerCase() == "true" || valStr.toLowerCase() == "1";
		}
		
	}
	
}